/*****************************************************************************
 *
 *  IntListParameter.cc                         XMM Science Analysis Subsystem
 *
 *  Integer-std::list task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: IntListParameter.cc,v 1.19 2002/12/17 17:53:49 jbakker Exp $
 *
 *****************************************************************************/

#include <sstream>             // C++ headers
#include <iostream>
#include <string>
#include <vector>
#include <list>

#include "IntListParameter.h"             // Application headers
#include "StringToken.h"
#include <errstr.h>
#include <StringFns.h>
#include <Evaluator.h>


//================================================================
//  Class IntListParameter
//================================================================

/*** Format the parameter value as a std::string in quotes ***/

std::string IntListParameter::asString() const
{
    std::ostringstream ostr;
    std::vector<int>::const_iterator it = _value.begin();
    if(it != _value.end())
        ostr << *it++;
    while(it != _value.end())
        ostr << ' ' << *it++;

    return ostr.str();
}


/*** Format the parameter value as a std::string in quotes ***/

std::string IntListParameter::defaultAsString() const
{
    std::ostringstream ostr;
    std::vector<int>::const_iterator it = _default.begin();

    if(it != _default.end())
        ostr << *it++;
    while(it != _default.end())
        ostr << ' ' << *it++;
    return ostr.str();
}


/***  Parse std::string representation and assign to 'value' if it is valid ***/

bool IntListParameter::parse(const std::string& str, std::vector<int>& value)
{
    std::list<std::string> stringTokens = StringFns::splitString(str);

    bool ok = true;
    std::list<std::string>::const_iterator it = stringTokens.begin();
    while(ok && it != stringTokens.end())
    {
        std::string s = *it++;
	StringToken t(s);
	int i = t.asInt();
	if(t.isValid())
	    value.push_back(i);
	else {
	  errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");
	  ok = false;
	}
    }
    return ok;
}



/*** Revert to default value ***/

void IntListParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether value is allowable for the parameter ***/

bool IntListParameter::isValid(const std::vector<int>& value) const
{
  Evaluator evaluator( constraints() );
  NumericSymbols ids = evaluator.numericIdentifiers();  
  if (ids.valuePtr.size() == 0) return evaluator.evalLogic();
      
  std::vector<int>::const_iterator it = value.begin();
  while(it != value.end()) {
    for (unsigned int i=0;i<ids.valuePtr.size();i++) *ids.valuePtr[i]=*it;
    if (!evaluator.evalLogic()) return false;
    it++;
  }
  return true;
}


/*** Set the parameter to the specified value ***/

bool IntListParameter::setValue(const std::vector<int>& value)
{
    bool valid = isValid(value);
    if(valid)
    {
        _value = value;
	setDefined();
	notifyView();
    }
    return valid;
}


/*** Set the default to the specified value ***/

bool IntListParameter::setDefault(const std::vector<int>& value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a std::string ***/

bool IntListParameter::setNewValue(const std::string& str)
{
    std::vector<int> values;
    bool ok =  parse(str, values) && setValue(values);

    if(!ok)
       reportRangeError(str);

    return true;
}


/***  Set the default, specified as a std::string ***/

bool IntListParameter::setNewDefault(const std::string& str)
{
    std::vector<int> values;
    bool ok = parse(str, values) && setDefault(values);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Return the length of the std::list ***/

int IntListParameter::size() const
{
    return _value.size();
}


/*** Report that value is out of range ***/

void IntListParameter::reportRangeError(const std::string& valueString)
{
  errstr << "Parameter '" << _name << "';" << std::endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}
