/*****************************************************************************
 *
 *  ParamInfoSet.cc                             XMM Science Analysis Subsystem
 *
 *  ParameterInfo for a set of tasks.
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ParamInfoSet.cc,v 1.8 2003/02/12 10:33:34 jbakker Exp $
 *
 *****************************************************************************/

#include "ParamInfoSet.h"            // Application headers
#include "ParameterInfo.h"
#include "XmlParamParser.h"
#include <errstr.h>
#include <SasFileLocator.h>

#include <iostream>             // C++ headers


//================================================================
//  Class ParamInfoSet [Singleton]
//================================================================

ParamInfoSet* ParamInfoSet::_infoSet = 0;

/* private */
ParamInfoSet::ParamInfoSet()
{
}

/*** Return the single instance [STATIC] ***/

ParamInfoSet* ParamInfoSet::instance()
{
    if(!_infoSet)
        _infoSet = new ParamInfoSet();

    return _infoSet;
}


/*** Revert to defaults for all tasks (simply clears cache) ***/

void ParamInfoSet::resetParamInfo()
{
  errstr << "Unloading all parameters for all tasks in memory"
	 << Sas::message(Msg::Lib, Msg::Noisy);

    ParamInfoMap::const_iterator it = _infoMap.begin();
    while(it != _infoMap.end())
    {
        ParameterInfo* p = (*it++).second;
	delete p;
    }
    _infoMap.clear();    
}


/*** Revert to defaults for specified task ***/

void ParamInfoSet::resetParamInfo(const std::string& taskName)
{
  errstr << "Unloading all parameters for task (" << taskName << ")"
	 << Sas::message(Msg::Lib, Msg::Noisy);

    ParamInfoMap::iterator it = _infoMap.find(taskName);
    if(it != _infoMap.end())
    {
        ParameterInfo* p = (*it).second;
	delete p;
	_infoMap.erase(it);
    }
}


/*** Get the parameter info for a specified task ***/

ParameterInfo* ParamInfoSet::getParamInfo(const std::string& taskName)
{
    ParameterInfo* info = 0;

    if(_infoMap.find(taskName) == _infoMap.end())
    {
        errstr << "Reading parameters from file for task(" << taskName << ")"
	       << Sas::message(Msg::Lib, Msg::Noisy);

	std::string searchName("config/" + taskName + ".par");
	SasFileLocator locator("");
	std::string fileName=locator.locate(searchName, "", false);
	if (fileName.empty())
	  errstr << "Could not find parameter file '"
		 << searchName << "' in SAS_PATH or SAS_DIR"
		 << Sas::error("ParamFileOpen");

        XmlParamParser parser;
	info=parser.load(fileName);
	if (info) {
	  info->setTaskName(taskName);
	  _infoMap[taskName] = info;
	}
    } else {
        errstr << "Reading cached parameters for task(" << taskName << ")"
	       << Sas::message(Msg::Lib, Msg::Noisy);

        info = _infoMap[taskName];     // Return cached value
    }

    return info;
}

#include <cstdlib> // for std::getenv

void ParamInfoSet::saveParamInfo(ParameterInfo* info)
{
  if (!info) return;

  char* home=std::getenv("HOME");
  if (!home) errstr << "Could not find HOME variable in your environment?"
		    << Sas::error("NotInEnvironment");

  std::string file(home);
  file+="/.sas.d/config/"+ info->taskName()+".par";
  
  XmlParamParser parser;
  parser.save(*info,file);
}
