/*****************************************************************************
 *
 *  RealParameter.cc                            XMM Science Analysis Subsystem
 *
 *  Real task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: RealParameter.cc,v 1.15 2002/12/17 17:53:49 jbakker Exp $
 *
 *****************************************************************************/

#include <sstream>             // C++ headers

#include "RealParameter.h"          // Application headers
#include "StringToken.h"
#include <errstr.h>
#include <iostream>
#include <Evaluator.h>


//================================================================
//  Class RealParameter
//================================================================


/*** Format the parameter value as a std::string ***/

std::string RealParameter::asString() const
{
    std::ostringstream ostr;
    ostr.precision(12);
    ostr << _value;
    return ostr.str();
}


/*** Format the default value as a std::string ***/

std::string RealParameter::defaultAsString() const
{
    std::ostringstream ostr;
    ostr.precision(12);
    ostr << _default;
    return ostr.str();
}


/***  Parse std::string representation and assign to 'value' if it is valid ***/

bool RealParameter::parse(const std::string& str, double& value)
{
    StringToken t(str);
    double n = t.asDouble();
    if(t.isValid())
        value = n;
    else
      errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");

    return true;
}


/*** Revert to default value ***/

void RealParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether value is allowable for the parameter ***/

bool RealParameter::isValid(double n) const
{
  Evaluator evaluator( constraints() );
  NumericSymbols ids = evaluator.numericIdentifiers();
  if (ids.valuePtr.size() == 0) return evaluator.evalLogic();

  for (unsigned int i=0;i<ids.valuePtr.size();i++) *ids.valuePtr[i]=n;
  return evaluator.evalLogic();
}


/*** Set the parameter to the specified value ***/

bool RealParameter::setValue(double value)
{
    bool valid = isValid(value);
    if(valid)
      {
        _value = value;
	setDefined();
	notifyView();
      }
    return valid;
}


/*** Set the default to the specified value ***/

bool RealParameter::setDefault(double value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a std::string ***/

bool RealParameter::setNewValue(const std::string& str)
{
    double value;
    bool ok = parse(str, value) && setValue(value);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Set the default, specified as a std::string ***/

bool RealParameter::setNewDefault(const std::string& str)
{
    double value;
    bool ok = parse(str, value) && setDefault(value);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Report that value is out of range ***/

void RealParameter::reportRangeError(const std::string& valueString)
{
  errstr << "Parameter '" << _name << "';" << std::endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}


