/*****************************************************************************
 *
 *  param_impl.cc                               XMM Science Analysis Subsystem
 *
 *  Implements F90 interface to parameter system
 *
 *  (c) 1998  ESA  -  Written by Marco Beijersbergen
 *
 *  $Id: param_impl.cc,v 1.22 2009/07/02 17:43:38 eojero Exp $
 *
 *****************************************************************************/


#include <cassert>		// C

#include <string>		// C++
#include <cstring>

#include <Tracer.h>		// SAS
#include <stringutil.h>
#include <f90cxx.h>

#include "param.h"


F90PROCDEF(Logical,booleanparameter)(
		const char *		name,
		const int *		pos,
		unsigned		nameLen )
{
  PROC
  bool b;
  if (pos)
    b = booleanParameter(std::string(name,nameLen), *pos);
  else
    b = booleanParameter(std::string(name,nameLen));
  RETURN b;
}

F90PROCDEF(int,intparameter)(
		const char *		name,
		const int *		pos,
		unsigned		nameLen )
{
  PROC
  int i;
  if (pos)
    i = intParameter(std::string(name,nameLen),*pos);
  else
    i = intParameter(std::string(name,nameLen));
  RETURN i;
}

F90PROCDEF(double,realparameter)(
		const char *		name,
		const int *		pos,
		unsigned		nameLen )
{
  PROC
  double d;
  if (pos)
    d = realParameter(std::string(name,nameLen),*pos);
  else
    d = realParameter(std::string(name,nameLen));
  RETURN d;
}


F90PROCDEF(void,stringparameter)(
		char *			val,
		unsigned 		valLen,
		const char *		name,
		const int *		pos,
		unsigned 		nameLen )
{
  PROC

  std::string param;

  if (pos) 
    param = stringParameter(std::string(name,nameLen),*pos);
  else
    param = stringParameter(std::string(name,nameLen));

  unsigned paramLen = param.length();
  unsigned n = std::min(valLen,paramLen);
  strncpy(val,param.c_str(),n);
  memset(val+n,' ',valLen-n);
  ENDPROC
}

F90PROCDEF(int,parametercount)(
		const char *		name,
		unsigned		nameLen )
{
  PROC
  RETURN parameterCount(std::string(name,nameLen));
}

F90PROCDEF(void,openparameters)(const char *category, unsigned categoryLen)
{
  PROC
  openParameters(std::string(category,categoryLen));
  ENDPROC
}

F90PROCDEF(void,closeparameters)()
{
  PROC
  closeParameters();
  ENDPROC
}

F90PROCDEF(void,setbooleanparameter)(const char* name, Logical& value, unsigned nameLen)
{
  PROC
  setParameter(std::string(name,nameLen),bool(value));
  ENDPROC
}

F90PROCDEF(void,setintparameter)(const char* name, int& value, unsigned nameLen)
{
  PROC
  setParameter(std::string(name,nameLen),value);
  ENDPROC
}

F90PROCDEF(void,setreal32parameter)(const char* name, float& value, unsigned nameLen)
{
  PROC
  setParameter(std::string(name,nameLen),value);
  ENDPROC
}

F90PROCDEF(void,setreal64parameter)(const char* name, double& value, unsigned nameLen)
{
  PROC
  setParameter(std::string(name,nameLen),value);
  ENDPROC
}

F90PROCDEF(void,setstringparameter)(const char* name, const char* value, unsigned nameLen, unsigned valueLen)
{
  PROC
  setParameter(std::string(name,nameLen),std::string(value,valueLen));
  ENDPROC
}

F90PROCDEF(void,setbooleanlistparameter)(const char* name, const Dope<Logical,1>& dope, unsigned nameLen)
{
  PROC
#ifdef 	NAGf90Fortran
  std::vector<Logical> value(
	dope.addr,
	dope.addr+dope.dim[0].extent);
#else
  std::vector<Logical> value(
	dope.addr,
	dope.addr+(dope.dim[0]._ubound+1-dope.dim[0].lower_bound));
#endif
  // Work-around for egcs compiler problem with vector<bool>
  std::vector<int> bvalue;
  for (unsigned i = 0; i < value.size(); ++i)
    bvalue.push_back(value[i]);
  setParameter(std::string(name,nameLen), bvalue );
  ENDPROC
}

F90PROCDEF(void,setintlistparameter)(const char* name, const Dope<int,1>& dope, unsigned nameLen)
{
  PROC
#ifdef 	NAGf90Fortran
  std::vector<int> value(dope.addr, dope.addr+dope.dim[0].extent);
#else
  std::vector<int> value(dope.addr, dope.addr+(dope.dim[0]._ubound+1-dope.dim[0].lower_bound));
#endif
  SAS_DEBUG( for (unsigned i = 0; i < value.size(); ++i) clog << value[i] << std::endl; )

  setParameter(std::string(name,nameLen), value);
  ENDPROC
}

F90PROCDEF(void,setreal64listparameter)(const char* name, const Dope<double,1>& dope, unsigned nameLen)
{
  PROC
#ifdef 	NAGf90Fortran
  std::vector<double> value(dope.addr, dope.addr+dope.dim[0].extent);
#else
  std::vector<double> value(dope.addr, dope.addr+(dope.dim[0]._ubound+1-dope.dim[0].lower_bound));
#endif
  setParameter(std::string(name,nameLen), value);
  ENDPROC
}

F90PROCDEF(void,setreal32listparameter)(const char* name, const Dope<float,1>& dope, unsigned nameLen)
{
  PROC
#ifdef NAGf90Fortran
  std::vector<float> value(dope.addr, dope.addr+dope.dim[0].extent);
#else
  std::vector<float> value(dope.addr, dope.addr+(dope.dim[0]._ubound+1-dope.dim[0].lower_bound));
#endif
  std::vector<double> fvalue;
  for (unsigned i = 0; i < value.size(); ++i) fvalue.push_back(value[i]);
  setParameter(std::string(name,nameLen), fvalue);
  ENDPROC
}


typedef ChDope<1> StringVector;
F90PROCDEF(void,setstringlistparameter)(const char* name, const ChDope<1>& dope, unsigned nameLen)
{
  PROC
  std::vector<std::string> value;
#ifdef NAGf90Fortran
  for (int i = 0; i < dope.dim[0].extent; ++i)
    value.push_back(trim(std::string(dope.addr+dope.dim[0].mult*i,dope.len)));
#else
  int size = dope.dtype >> 6;
  for (int i = 0; i <(dope.dim[0]._ubound+1-dope.dim[0].lower_bound); ++i)
     value.push_back(trim(std::string(dope.addr+dope.dim[0]._stride*i*size,size)));
#endif
  setParameter(std::string(name,nameLen),value);
  ENDPROC
}

